--[[
Get Image Size
By: MikuAuahDark
Allows you to get image size.
Supported Image/Video Files(by priority order):
1. Portable Network Graphics
2. Windows Bitmap
3. JPEG
]]
function GetImageWidthHeight(file)
     local fileinfo=type(file)
     if type(file)=="string" then
          file=assert(io.open(file,"rb"))
     else
          fileinfo=file:seek("cur")
     end
     local function refresh()
          if type(fileinfo)=="number" then
               file:seek("set",fileinfo)
          else
               file:close()
          end
     end
     local width,height=0,0
     file:seek("set",1)
     -- Detect if PNG
     if file:read(3)=="PNG" then
          --[[
               The strategy is
               1. Seek to position 0x10
               2. Get value in big-endian order
          ]]
          file:seek("set",16)
          local widthstr,heightstr=file:read(4),file:read(4)
          if type(fileinfo)=="number" then
               file:seek("set",fileinfo)
          else
               file:close()
          end
          width=widthstr:sub(1,1):byte()*16777216+widthstr:sub(2,2):byte()*65536+widthstr:sub(3,3):byte()*256+widthstr:sub(4,4):byte()
          height=heightstr:sub(1,1):byte()*16777216+heightstr:sub(2,2):byte()*65536+heightstr:sub(3,3):byte()*256+heightstr:sub(4,4):byte()
          return width,height
     end
     file:seek("set")
     -- Detect if BMP
     if file:read(2)=="BM" then
          --[[ 
               The strategy is:
               1. Seek to position 0x12
               2. Get value in little-endian order
          ]]
          file:seek("set",18)
          local widthstr,heightstr=file:read(4),file:read(4)
          refresh()
          width=widthstr:sub(4,4):byte()*16777216+widthstr:sub(3,3):byte()*65536+widthstr:sub(2,2):byte()*256+widthstr:sub(1,1):byte()
          height=heightstr:sub(4,4):byte()*16777216+heightstr:sub(3,3):byte()*65536+heightstr:sub(2,2):byte()*256+heightstr:sub(1,1):byte()
          return width,height
     end
     -- Detect if JPG/JPEG
     file:seek("set")
     if file:read(2)=="\255\216" then
          --[[
               The strategy is
               1. Find necessary markers
               2. Store biggest value in variable
               3. Return biggest value
          ]]
          local lastb,curb=0,0
          local xylist={}
          local sstr=file:read(1)
          while sstr~=nil do
               lastb=curb
               curb=sstr:byte()
               if (curb==194 or curb==192) and lastb==255 then
                    file:seek("cur",3)
                    local sizestr=file:read(4)
                    local h=sizestr:sub(1,1):byte()*256+sizestr:sub(2,2):byte()
                    local w=sizestr:sub(3,3):byte()*256+sizestr:sub(4,4):byte()
                    if w>width and h>height then
                         width=w
                         height=h
                    end
               end
               sstr=file:read(1)
          end
          if width>0 and height>0 then
               refresh()
               return width,height
          end
     end
     refresh()
     return nil
end