editor = {
	data = {
		tileset = {
			image = {}
		},
		menu = {},
		player = {}
	},
	hook = {}
}
editor.data.tileset.image.width, editor.data.tileset.image.height = GetImageWidthHeight("gfx/tiles/" .. map("tileset"))
local file = io.open("maps/" .. map("name") .. ".map", "rb")

if (file:read("*l") == "Unreal Software's Counter-Strike 2D Map File" .. " (max)\r") then
	file:read(50)
	
	for rep = 1, 12 do
		file:read("*l")
	end
	
	file:read(9)
	file:read("*l")
	file:read(11)
	
	if (file:read("*l") == "ed.erawtfoslaernu\r") then
		for index = 1, map("tilecount") + 1 do
			editor.data.tileset[index] = file:read(1):byte()
		end
	end
end

function editor.resetplayerdata(id)
	editor.data.player[id] = {
		clientdata = {},
		drawing = false,
		draw = {
			mode = 0,
			frame = {0, 0},
			pos = {
				{
					x = 0,
					y = 0
				},
				{
					x = 0,
					y = 0
				},
				{
					x = 0,
					y = 0
				},
				frame = {}
			}
		},
		menu = {
			page = 0,
			button = 0
		},
		image = {},
		history = {
			undo = {
				count = 0
			},
			redo = {
				count = 0
			}
		}
	}
end

function editor.menu(id, title, ...)
	if (editor.data.menu[title]) then
		if (#{...} > 0) then
			local buttons = {...}
			local samebuttons = true
			
			for index, button in pairs(buttons) do
				if (editor.data.menu[title].button[index] ~= button) then
					samebuttons = false
					
					break
				end
			end
			
			if (samebuttons) then
				return menu(id, editor.data.menu[title].page[editor.data.player[id].menu.page])
			end
		else
			return menu(id, editor.data.menu[title].page[editor.data.player[id].menu.page])
		end
	end
	
	editor.data.menu[title] = {button = {...}, page = {}}
	editor.data.menu[title].pages = #editor.data.menu[title].button > 9 and math.ceil(#editor.data.menu[title].button / 6) or 1
	local selectionto, button = 0, 0
	
	for page = 1, editor.data.menu[title].pages do
		editor.data.menu[title].page[page] = title .. ","
		selectionto = editor.data.menu[title].pages == 1 and 9 or 6
		
		for selection = 1, selectionto do
			button = editor.data.menu[title].pages > 1 and selection + (6 * (page - 1)) or selection
			
			if (editor.data.menu[title].button[button]) then
				editor.data.menu[title].page[page] = editor.data.menu[title].page[page] .. editor.data.menu[title].button[button] .. ","
			else
				editor.data.menu[title].page[page] = editor.data.menu[title].page[page] .. ","
			end
		end
		
		if (editor.data.menu[title].pages > 1) then
			if (page == 1) then
				editor.data.menu[title].page[page] = editor.data.menu[title].page[page] .. ",(Previous)"
			else
				editor.data.menu[title].page[page] = editor.data.menu[title].page[page] .. ",Previous"
			end
			
			if (page == editor.data.menu[title].pages) then
				editor.data.menu[title].page[page] = editor.data.menu[title].page[page] .. ",(Next)"
			else
				editor.data.menu[title].page[page] = editor.data.menu[title].page[page] .. ",Next"
			end
		end
	end
	
	editor.data.player[id].menu.page = 1
	
	return menu(id, editor.data.menu[title].page[editor.data.player[id].menu.page])
end

function editor.mainmenu(id)
	return editor.menu(id, "Map Tile Editor", "Draw Tile by Tile", "Draw Area of Tiles", "Fill Area with Tiles", "Replace Tiles with Tile", "Copy Area of Tiles")
end

function editor.tilemode(mode)
	if (mode == 0) then
		return "floor (no sound)"
	elseif (mode == 1) then
		return "wall"
	elseif (mode == 2) then
		return "obstacle"
	elseif (mode == 3) then
		return "wall (no shadow)"
	elseif (mode == 4) then
		return "obstacle (no shadow)"
	elseif (mode == 5) then
		return "wall (floor level)"
	elseif (mode == 10) then
		return "floor dirt"
	elseif (mode == 11) then
		return "floor snow (footprints fx)"
	elseif (mode == 12) then
		return "floor step"
	elseif (mode == 13) then
		return "floor tile"
	elseif (mode == 14) then
		return "floor water (wave fx)"
	elseif (mode == 15) then
		return "floor metal"
	elseif (mode == 16) then
		return "floor wood"
	elseif (mode == 50) then
		return "deadly normal"
	elseif (mode == 51) then
		return "deadly toxic"
	elseif (mode == 52) then
		return "deadly explosion"
	elseif (mode == 53) then
		return "deadly abyss"
	end
end

function editor.tilemenu(id, title)
	editor.data.player[id].image.tileset = image("gfx/tiles/" .. map("tileset") .. "<m>", 0, 0, 2, id)
	local imagewidth, imageheight = editor.data.tileset.image.width, editor.data.tileset.image.height
	local scale = 1
	
	if (imagewidth > 280) then
		scale = imagewidth * (280 / imagewidth) > 440 and 440 / imageheight or 280 / imagewidth
	end
	
	if (imageheight > 440) then
		scale = imagewidth * (440 / imageheight) > 280 and 280 / imagewidth or 440 / imageheight
	end
	
	imagescale(editor.data.player[id].image.tileset, scale, scale)
	imagepos(editor.data.player[id].image.tileset, 480, 240, 0)
	
	local indexstring = {}
	
	for index = 1, #editor.data.tileset do
		indexstring[#indexstring + 1] = tostring(index - 1) .. "|" .. editor.tilemode(editor.data.tileset[index])
	end
	
	return editor.menu(id, title, unpack(indexstring))
end

function editor.optionsmenu(id)
	if (editor.data.player[id].draw.mode <= 4) then
		return editor.menu(id, "Options", "Reset Frame", "Reset Map", "", editor.data.player[id].history.undo.count > 0 and "Undo" or "(Undo)", editor.data.player[id].history.redo.count > 0 and "Redo" or "(Redo)")
	elseif (editor.data.player[id].draw.mode == 8) then
		return editor.menu(id, "Options", "Reset Area", "Reset Map", "", editor.data.player[id].history.undo.count > 0 and "Undo" or "(Undo)", editor.data.player[id].history.redo.count > 0 and "Redo" or "(Redo)")
	else
		return editor.menu(id, "Options", "Reset Map", "", editor.data.player[id].history.undo.count > 0 and "Undo" or "(Undo)", editor.data.player[id].history.redo.count > 0 and "Redo" or "(Redo)")
	end
end

function editor.clientdata(id, mode, parameter)
	reqcld(id, mode, parameter)
	
	return editor.data.player[id].clientdata[mode][1], editor.data.player[id].clientdata[mode][2]
end

function editor.totile(number)
	return math.floor(number / 32)
end

function editor.getstep(number1, number2, step)
	step = step or 1
	
	return number1 > number2 and -step or step
end

function editor.removeblock(id)
	if (editor.data.player[id].image.block) then
		freeimage(editor.data.player[id].image.block)
		
		editor.data.player[id].image.block = nil
	end
end

function editor.drawblock(id)
	local tilex, tiley = editor.clientdata(id, 2)
	tilex, tiley = editor.totile(tilex), editor.totile(tiley)
	
	editor.removeblock(id)
	
	editor.data.player[id].image.block = image("gfx/block.bmp", 0, 0, 1, id)
	
	imagecolor(editor.data.player[id].image.block, 255, 255, 0)
	imagealpha(editor.data.player[id].image.block, 0.5)
	
	if (editor.data.player[id].draw.mode ~= 3 and editor.data.player[id].draw.mode <= 6) then
		imagepos(editor.data.player[id].image.block, tilex * 32 + 16, tiley * 32 + 16, 0)
	else
		if (editor.data.player[id].draw.mode ~= 8) then
			imagescale(editor.data.player[id].image.block, math.abs(editor.data.player[id].draw.pos[1].x - tilex) + 1, math.abs(editor.data.player[id].draw.pos[1].y - tiley) + 1)
			imagepos(editor.data.player[id].image.block, editor.data.player[id].draw.pos[1].x * 32 + (tilex - editor.data.player[id].draw.pos[1].x + 1) * 32 / 2, editor.data.player[id].draw.pos[1].y * 32 + (tiley - editor.data.player[id].draw.pos[1].y + 1) * 32 / 2, 0)
		else
			imagescale(editor.data.player[id].image.block, math.abs(editor.data.player[id].draw.pos[2].x - editor.data.player[id].draw.pos[1].x) + 1, math.abs(editor.data.player[id].draw.pos[2].y - editor.data.player[id].draw.pos[1].y) + 1)
			imagepos(editor.data.player[id].image.block, tilex * 32 + (math.abs(editor.data.player[id].draw.pos[2].x - editor.data.player[id].draw.pos[1].x) + 1) * 32 / 2, tiley * 32 + (math.abs(editor.data.player[id].draw.pos[2].y - editor.data.player[id].draw.pos[1].y) + 1) * 32 / 2, 0)
		end
	end
end

function editor.getfromto(number1, number2)
	if (number1 <= number2) then
		return number1, number2
	else
		return number2, number1
	end
end

function editor.settile(id)
	if (editor.data.player[id].draw.mode > 0) then
		local tilex, tiley = editor.clientdata(id, 2)
		tilex, tiley = editor.totile(tilex), editor.totile(tiley)
		
		if (editor.data.player[id].draw.mode ~= 3 and editor.data.player[id].draw.mode <= 6) then
			editor.data.player[id].draw.pos[1].x = tilex
			editor.data.player[id].draw.pos[1].y = tiley
		else
			if (editor.data.player[id].draw.mode ~= 8) then
				editor.data.player[id].draw.pos[2].x = tilex
				editor.data.player[id].draw.pos[2].y = tiley
			else
				editor.data.player[id].draw.pos[3].x = tilex
				editor.data.player[id].draw.pos[3].y = tiley
			end
		end
		
		if (editor.data.player[id].draw.mode <= 3) then
			if (editor.data.player[id].draw.mode ~= 2) then
				if (editor.data.player[id].draw.mode == 1) then
					parse("settile " .. editor.data.player[id].draw.pos[1].x .. " " .. editor.data.player[id].draw.pos[1].y .. " " .. editor.data.player[id].draw.frame[1])
				elseif (editor.data.player[id].draw.mode == 3) then
					for tilex = editor.data.player[id].draw.pos[1].x, editor.data.player[id].draw.pos[2].x, editor.getstep(editor.data.player[id].draw.pos[1].x, editor.data.player[id].draw.pos[2].x) do
						for tiley = editor.data.player[id].draw.pos[1].y, editor.data.player[id].draw.pos[2].y, editor.getstep(editor.data.player[id].draw.pos[1].y, editor.data.player[id].draw.pos[2].y) do
							parse("settile " .. tilex .. " " .. tiley .. " " .. editor.data.player[id].draw.frame[1])
						end
					end
					
					editor.data.player[id].draw.mode = 2
				end
			else
				editor.data.player[id].draw.mode = 3
			end
		elseif (editor.data.player[id].draw.mode == 4) then
			local fillframe = tile(editor.data.player[id].draw.pos[1].x, editor.data.player[id].draw.pos[1].y, "frame")
			
			if (fillframe ~= editor.data.player[id].draw.frame[1]) then
				local lasttime = os.time()
				local tilecount = 1
				local filltiles = {
					{
						x = editor.data.player[id].draw.pos[1].x,
						y = editor.data.player[id].draw.pos[1].y
					}
				}
				
				while (tilecount > 0) do
					for index, filltile in pairs(filltiles) do
						parse("settile " .. filltile.x .. " " .. filltile.y .. " " .. editor.data.player[id].draw.frame[1])
						
						tilecount = tilecount - 1
						
						if (filltile.x - 1 >= 0) then
							if (tile(filltile.x - 1, filltile.y, "frame") == fillframe) then
								parse("settile " .. filltile.x - 1 .. " " .. filltile.y .. " " .. editor.data.player[id].draw.frame[1])
								table.insert(filltiles, {
									x = filltile.x - 1,
									y = filltile.y
								})
								
								tilecount = tilecount + 1
							end
						end
						
						if (filltile.x + 1 >= 0) then
							if (tile(filltile.x + 1, filltile.y, "frame") == fillframe) then
								parse("settile " .. filltile.x + 1 .. " " .. filltile.y .. " " .. editor.data.player[id].draw.frame[1])
								table.insert(filltiles, {
									x = filltile.x + 1,
									y = filltile.y
								})
								
								tilecount = tilecount + 1
							end
						end
						
						if (filltile.y - 1 >= 0) then
							if (tile(filltile.x, filltile.y - 1, "frame") == fillframe) then
								parse("settile " .. filltile.x .. " " .. filltile.y - 1 .. " " .. editor.data.player[id].draw.frame[1])
								table.insert(filltiles, {
									x = filltile.x,
									y = filltile.y - 1
								})
								
								tilecount = tilecount + 1
							end
						end
						
						if (filltile.y + 1 >= 0) then
							if (tile(filltile.x, filltile.y + 1, "frame") == fillframe) then
								parse("settile " .. filltile.x .. " " .. filltile.y + 1 .. " " .. editor.data.player[id].draw.frame[1])
								table.insert(filltiles, {
									x = filltile.x,
									y = filltile.y + 1
								})
								
								tilecount = tilecount + 1
							end
						end
				
						if (os.time() - lasttime > 3000) then
							break
						end
					end
				end
			end
		elseif (editor.data.player[id].draw.mode == 5) then
			for tilex = 0, map("xsize") do
				for tiley = 0, map("ysize") do
					if (tile(tilex, tiley, "frame") == editor.data.player[id].draw.frame[1]) then
						parse("settile " .. tilex .. " " .. tiley .. " " .. editor.data.player[id].draw.frame[2])
					end
				end
			end
			
			editor.data.player[id].draw.mode = 0
		elseif (editor.data.player[id].draw.mode == 6 or editor.data.player[id].draw.mode == 7) then
			if (editor.data.player[id].draw.mode == 7) then
				local countx, county = 0, 0
				local fromx, tox = editor.getfromto(editor.data.player[id].draw.pos[1].x, editor.data.player[id].draw.pos[2].x)
				local fromy, toy = editor.getfromto(editor.data.player[id].draw.pos[1].y, editor.data.player[id].draw.pos[2].y)
				
				for tilex = fromx, tox do
					countx = countx + 1
					editor.data.player[id].draw.pos.frame[countx] = {}
					
					for tiley = fromy, toy do
						county = county + 1
						editor.data.player[id].draw.pos.frame[countx][county] = tile(tilex, tiley, "frame")
					end
					
					county = 0
				end
			end
			
			editor.data.player[id].draw.mode = editor.data.player[id].draw.mode == 6 and 7 or 8
		elseif (editor.data.player[id].draw.mode == 8) then
			local scalex, scaley = math.abs(editor.data.player[id].draw.pos[2].x - editor.data.player[id].draw.pos[1].x), math.abs(editor.data.player[id].draw.pos[2].y - editor.data.player[id].draw.pos[1].y)
			local countx, county = 0, 0
			
			for tilex = editor.data.player[id].draw.pos[3].x, editor.data.player[id].draw.pos[3].x + scalex do
				countx = countx + 1
				
				for tiley = editor.data.player[id].draw.pos[3].y, editor.data.player[id].draw.pos[3].y + scaley do
					county = county + 1
					
					parse("settile " .. tilex .. " " .. tiley .. " " .. editor.data.player[id].draw.pos.frame[countx][county])
				end
				
				county = 0
			end
		end
	end
end

function editor.newhistory(id, type)
	editor.data.player[id].history[type].count = editor.data.player[id].history[type].count + 1
	editor.data.player[id].history[type][editor.data.player[id].history[type].count] = {}
	
	for tilex = 0, map("xsize") do
		editor.data.player[id].history[type][editor.data.player[id].history[type].count][tilex + 1] = {}
		
		for tiley = 0, map("ysize") do
			editor.data.player[id].history[type][editor.data.player[id].history[type].count][tilex + 1][tiley + 1] = tile(tilex, tiley, "frame")
		end
	end
end

function editor.removehistory(id, type, all)
	if (editor.data.player[id].history[type].count > 0) then
		if (all) then
			for count = 1, editor.data.player[id].history[type].count do
				editor.data.player[id].history[type][count] = nil
			end
			
			editor.data.player[id].history[type].count = 0
		else
			editor.data.player[id].history[type][editor.data.player[id].history[type].count] = nil
			editor.data.player[id].history[type].count = editor.data.player[id].history[type].count - 1
		end
	end
end

function editor.sethistory(id, type1)
	type2 = type1 == "undo" and "redo" or "undo"
	
	editor.newhistory(id, type2)
	
	for tilex = 0, map("xsize") do
		for tiley = 0, map("ysize")  do
			if (tile(tilex, tiley, "frame") ~= editor.data.player[id].history[type1][editor.data.player[id].history[type1].count][tilex + 1][tiley + 1]) then
				parse("settile " .. tilex .. " " .. tiley .. " " .. editor.data.player[id].history[type1][editor.data.player[id].history[type1].count][tilex + 1][tiley + 1])
			end
		end
	end
	
	editor.removehistory(id, type1)
end

function editor.resetmap()
	for tilex = 0, map("xsize") do
		for tiley = 0, map("ysize") do
			if (tile(tilex, tiley, "frame") ~= tile(tilex, tiley, "originalframe")) then
				parse("settile " .. tilex .. " " .. tiley .. " " .. tile(tilex, tiley, "originalframe"))
			end
		end
	end
end